@@ -1,11 +1,11 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 from pyasn1.compat.octets import null
-from pysnmp.carrier.asyncore.dgram import udp, udp6, unix
+from pysnmp.carrier.asyncore.dgram import udp, udp6
 from pysnmp.proto.secmod.rfc3414.auth import hmacmd5, hmacsha, noauth
 from pysnmp.proto.secmod.rfc3414.priv import des, nopriv
 from pysnmp.proto.secmod.rfc3826.priv import aes
@@ -19,7 +19,6 @@ from pysnmp import error
 # Transports
 snmpUDPDomain = udp.snmpUDPDomain
 snmpUDP6Domain = udp6.snmpUDP6Domain
-snmpLocalDomain = unix.snmpLocalDomain
 
 # Auth protocol
 usmHMACMD5AuthProtocol = hmacmd5.HmacMd5.serviceID
@@ -84,17 +83,19 @@ def addV1System(snmpEngine, communityIndex, communityName,
         contextName = null
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpCommunityEntry.name + (8,) + tblIdx, 'destroy'),)
+        (snmpCommunityEntry.name + (8,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpCommunityEntry.name + (1,) + tblIdx, communityIndex),
-         (snmpCommunityEntry.name + (2,) + tblIdx, communityName),
-         (snmpCommunityEntry.name + (3,) + tblIdx, securityName is not None and securityName or communityIndex),
-         (snmpCommunityEntry.name + (4,) + tblIdx, contextEngineId),
-         (snmpCommunityEntry.name + (5,) + tblIdx, contextName),
-         (snmpCommunityEntry.name + (6,) + tblIdx, transportTag),
-         (snmpCommunityEntry.name + (7,) + tblIdx, 'nonVolatile'),
-         (snmpCommunityEntry.name + (8,) + tblIdx, 'createAndGo'))
+        (snmpCommunityEntry.name + (1,) + tblIdx, communityIndex),
+        (snmpCommunityEntry.name + (2,) + tblIdx, communityName),
+        (snmpCommunityEntry.name + (3,) + tblIdx, securityName is not None and securityName or communityIndex),
+        (snmpCommunityEntry.name + (4,) + tblIdx, contextEngineId),
+        (snmpCommunityEntry.name + (5,) + tblIdx, contextName),
+        (snmpCommunityEntry.name + (6,) + tblIdx, transportTag),
+        (snmpCommunityEntry.name + (7,) + tblIdx, 'nonVolatile'),
+        (snmpCommunityEntry.name + (8,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -102,7 +103,8 @@ def delV1System(snmpEngine, communityIndex):
     (snmpCommunityEntry, tblIdx,
      snmpEngineID) = __cookV1SystemInfo(snmpEngine, communityIndex)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpCommunityEntry.name + (8,) + tblIdx, 'destroy'),)
+        (snmpCommunityEntry.name + (8,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -129,15 +131,11 @@ def addV3User(snmpEngine, userName,
               authProtocol=usmNoAuthProtocol, authKey=None,
               privProtocol=usmNoPrivProtocol, privKey=None,
               securityEngineId=None,
-              securityName=None,
-              # deprecated parameters follow
-              contextEngineId=None):
+              securityName=None):
     mibBuilder = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder
 
     if securityName is None:
         securityName = userName
-    if securityEngineId is None:  # backward compatibility
-        securityEngineId = contextEngineId
     (snmpEngineID, usmUserEntry, tblIdx1,
      pysnmpUsmSecretEntry, tblIdx2) = __cookV3UserInfo(snmpEngine, userName, securityEngineId)
 
@@ -148,15 +146,17 @@ def addV3User(snmpEngine, userName,
     zeroDotZero, = mibBuilder.importSymbols('SNMPv2-SMI', 'zeroDotZero')
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((usmUserEntry.name + (13,) + tblIdx1, 'destroy'),)
+        (usmUserEntry.name + (13,) + tblIdx1, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((usmUserEntry.name + (2,) + tblIdx1, userName),
-         (usmUserEntry.name + (3,) + tblIdx1, securityName),
-         (usmUserEntry.name + (4,) + tblIdx1, zeroDotZero.name),
-         (usmUserEntry.name + (5,) + tblIdx1, authProtocol),
-         (usmUserEntry.name + (8,) + tblIdx1, privProtocol),
-         (usmUserEntry.name + (13,) + tblIdx1, 'createAndGo'))
+        (usmUserEntry.name + (2,) + tblIdx1, userName),
+        (usmUserEntry.name + (3,) + tblIdx1, securityName),
+        (usmUserEntry.name + (4,) + tblIdx1, zeroDotZero.name),
+        (usmUserEntry.name + (5,) + tblIdx1, authProtocol),
+        (usmUserEntry.name + (8,) + tblIdx1, privProtocol),
+        (usmUserEntry.name + (13,) + tblIdx1, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     # Localize keys
@@ -182,39 +182,40 @@ def addV3User(snmpEngine, userName,
 
     # Commit localized keys
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((pysnmpUsmKeyEntry.name + (1,) + tblIdx1, localAuthKey),
-         (pysnmpUsmKeyEntry.name + (2,) + tblIdx1, localPrivKey),
-         (pysnmpUsmKeyEntry.name + (3,) + tblIdx1, hashedAuthPassphrase),
-         (pysnmpUsmKeyEntry.name + (4,) + tblIdx1, hashedPrivPassphrase))
+        (pysnmpUsmKeyEntry.name + (1,) + tblIdx1, localAuthKey),
+        (pysnmpUsmKeyEntry.name + (2,) + tblIdx1, localPrivKey),
+        (pysnmpUsmKeyEntry.name + (3,) + tblIdx1, hashedAuthPassphrase),
+        (pysnmpUsmKeyEntry.name + (4,) + tblIdx1, hashedPrivPassphrase),
+        **dict(snmpEngine=snmpEngine)
     )
 
     # Commit passphrases
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'destroy'),)
+        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((pysnmpUsmSecretEntry.name + (1,) + tblIdx2, userName),
-         (pysnmpUsmSecretEntry.name + (2,) + tblIdx2, authKey),
-         (pysnmpUsmSecretEntry.name + (3,) + tblIdx2, privKey),
-         (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'createAndGo'))
+        (pysnmpUsmSecretEntry.name + (1,) + tblIdx2, userName),
+        (pysnmpUsmSecretEntry.name + (2,) + tblIdx2, authKey),
+        (pysnmpUsmSecretEntry.name + (3,) + tblIdx2, privKey),
+        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
 def delV3User(snmpEngine,
               userName,
-              securityEngineId=None,
-              # deprecated parameters follow
-              contextEngineId=None):
-    if securityEngineId is None:  # backward compatibility
-        securityEngineId = contextEngineId
+              securityEngineId=None):
     (snmpEngineID, usmUserEntry, tblIdx1, pysnmpUsmSecretEntry,
      tblIdx2) = __cookV3UserInfo(snmpEngine, userName, securityEngineId)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((usmUserEntry.name + (13,) + tblIdx1, 'destroy'),)
+        (usmUserEntry.name + (13,) + tblIdx1, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'destroy'),)
+        (pysnmpUsmSecretEntry.name + (4,) + tblIdx2, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     # Drop all derived rows
@@ -225,7 +226,7 @@ def delV3User(snmpEngine,
     )
     while varBinds:
         varBinds = snmpEngine.msgAndPduDsp.mibInstrumController.readNextVars(
-            varBinds
+            *varBinds, **dict(snmpEngine=snmpEngine)
         )
         if varBinds[0][1].isSameTypeWith(rfc1905.endOfMibView):
             break
@@ -258,22 +259,25 @@ def addTargetParams(snmpEngine, name, securityName, securityLevel, mpModel=3):
     snmpTargetParamsEntry, tblIdx = __cookTargetParamsInfo(snmpEngine, name)
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetParamsEntry.name + (7,) + tblIdx, 'destroy'),)
+        (snmpTargetParamsEntry.name + (7,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetParamsEntry.name + (1,) + tblIdx, name),
-         (snmpTargetParamsEntry.name + (2,) + tblIdx, mpModel),
-         (snmpTargetParamsEntry.name + (3,) + tblIdx, securityModel),
-         (snmpTargetParamsEntry.name + (4,) + tblIdx, securityName),
-         (snmpTargetParamsEntry.name + (5,) + tblIdx, securityLevel),
-         (snmpTargetParamsEntry.name + (7,) + tblIdx, 'createAndGo'))
+        (snmpTargetParamsEntry.name + (1,) + tblIdx, name),
+        (snmpTargetParamsEntry.name + (2,) + tblIdx, mpModel),
+        (snmpTargetParamsEntry.name + (3,) + tblIdx, securityModel),
+        (snmpTargetParamsEntry.name + (4,) + tblIdx, securityName),
+        (snmpTargetParamsEntry.name + (5,) + tblIdx, securityLevel),
+        (snmpTargetParamsEntry.name + (7,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
 def delTargetParams(snmpEngine, name):
     snmpTargetParamsEntry, tblIdx = __cookTargetParamsInfo(snmpEngine, name)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetParamsEntry.name + (7,) + tblIdx, 'destroy'),)
+        (snmpTargetParamsEntry.name + (7,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -308,18 +312,20 @@ def addTargetAddr(snmpEngine, addrName, transportDomain, transportAddress,
         sourceAddress = TransportAddressIPv6(sourceAddress)
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetAddrEntry.name + (9,) + tblIdx, 'destroy'),)
+        (snmpTargetAddrEntry.name + (9,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetAddrEntry.name + (1,) + tblIdx, addrName),
-         (snmpTargetAddrEntry.name + (2,) + tblIdx, transportDomain),
-         (snmpTargetAddrEntry.name + (3,) + tblIdx, transportAddress),
-         (snmpTargetAddrEntry.name + (4,) + tblIdx, timeout),
-         (snmpTargetAddrEntry.name + (5,) + tblIdx, retryCount),
-         (snmpTargetAddrEntry.name + (6,) + tblIdx, tagList),
-         (snmpTargetAddrEntry.name + (7,) + tblIdx, params),
-         (snmpSourceAddrEntry.name + (1,) + tblIdx, sourceAddress),
-         (snmpTargetAddrEntry.name + (9,) + tblIdx, 'createAndGo'))
+        (snmpTargetAddrEntry.name + (1,) + tblIdx, addrName),
+        (snmpTargetAddrEntry.name + (2,) + tblIdx, transportDomain),
+        (snmpTargetAddrEntry.name + (3,) + tblIdx, transportAddress),
+        (snmpTargetAddrEntry.name + (4,) + tblIdx, timeout),
+        (snmpTargetAddrEntry.name + (5,) + tblIdx, retryCount),
+        (snmpTargetAddrEntry.name + (6,) + tblIdx, tagList),
+        (snmpTargetAddrEntry.name + (7,) + tblIdx, params),
+        (snmpSourceAddrEntry.name + (1,) + tblIdx, sourceAddress),
+        (snmpTargetAddrEntry.name + (9,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -327,7 +333,8 @@ def delTargetAddr(snmpEngine, addrName):
     (snmpTargetAddrEntry, snmpSourceAddrEntry,
      tblIdx) = __cookTargetAddrInfo(snmpEngine, addrName)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpTargetAddrEntry.name + (9,) + tblIdx, 'destroy'),)
+        (snmpTargetAddrEntry.name + (9,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -389,13 +396,31 @@ delSocketTransport = delTransport
 
 # VACM shortcuts
 
-def addContext(snmpEngine, contextName):
+def __cookVacmContextInfo(snmpEngine, contextName):
     mibBuilder = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder
-
     vacmContextEntry, = mibBuilder.importSymbols('SNMP-VIEW-BASED-ACM-MIB', 'vacmContextEntry')
     tblIdx = vacmContextEntry.getInstIdFromIndices(contextName)
+    return vacmContextEntry, tblIdx
+
+
+def addContext(snmpEngine, contextName):
+    vacmContextEntry, tblIdx = __cookVacmContextInfo(snmpEngine, contextName)
+
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmContextEntry.name + (1,) + tblIdx, contextName),)
+        (vacmContextEntry.name + (2,) + tblIdx, 'destroy')
+    )
+    snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
+        (vacmContextEntry.name + (1,) + tblIdx, contextName),
+        (vacmContextEntry.name + (2,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
+    )
+
+
+def delContext(snmpEngine, contextName):
+    vacmContextEntry, tblIdx = __cookVacmContextInfo(snmpEngine, contextName)
+
+    snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
+        (vacmContextEntry.name + (2,) + tblIdx, 'destroy')
     )
 
 
@@ -413,13 +438,15 @@ def addVacmGroup(snmpEngine, groupName, securityModel, securityName):
     (vacmSecurityToGroupEntry,
      tblIdx) = __cookVacmGroupInfo(snmpEngine, securityModel, securityName)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'destroy'),)
+        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmSecurityToGroupEntry.name + (1,) + tblIdx, securityModel),
-         (vacmSecurityToGroupEntry.name + (2,) + tblIdx, securityName),
-         (vacmSecurityToGroupEntry.name + (3,) + tblIdx, groupName),
-         (vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'createAndGo'))
+        (vacmSecurityToGroupEntry.name + (1,) + tblIdx, securityModel),
+        (vacmSecurityToGroupEntry.name + (2,) + tblIdx, securityName),
+        (vacmSecurityToGroupEntry.name + (3,) + tblIdx, groupName),
+        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -428,7 +455,8 @@ def delVacmGroup(snmpEngine, securityModel, securityName):
         snmpEngine, securityModel, securityName
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'destroy'),)
+        (vacmSecurityToGroupEntry.name + (5,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -448,20 +476,22 @@ def addVacmAccess(snmpEngine, groupName, contextName, securityModel,
                                                    contextName, securityModel,
                                                    securityLevel)
 
-    addContext(snmpEngine, contextName)  # this is leaky
+    addContext(snmpEngine, contextName)
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmAccessEntry.name + (9,) + tblIdx, 'destroy'),)
+        (vacmAccessEntry.name + (9,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmAccessEntry.name + (1,) + tblIdx, contextName),
-         (vacmAccessEntry.name + (2,) + tblIdx, securityModel),
-         (vacmAccessEntry.name + (3,) + tblIdx, securityLevel),
-         (vacmAccessEntry.name + (4,) + tblIdx, prefix),
-         (vacmAccessEntry.name + (5,) + tblIdx, readView),
-         (vacmAccessEntry.name + (6,) + tblIdx, writeView),
-         (vacmAccessEntry.name + (7,) + tblIdx, notifyView),
-         (vacmAccessEntry.name + (9,) + tblIdx, 'createAndGo'))
+        (vacmAccessEntry.name + (1,) + tblIdx, contextName),
+        (vacmAccessEntry.name + (2,) + tblIdx, securityModel),
+        (vacmAccessEntry.name + (3,) + tblIdx, securityLevel),
+        (vacmAccessEntry.name + (4,) + tblIdx, prefix),
+        (vacmAccessEntry.name + (5,) + tblIdx, readView),
+        (vacmAccessEntry.name + (6,) + tblIdx, writeView),
+        (vacmAccessEntry.name + (7,) + tblIdx, notifyView),
+        (vacmAccessEntry.name + (9,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -470,8 +500,12 @@ def delVacmAccess(snmpEngine, groupName, contextName, securityModel,
     vacmAccessEntry, tblIdx = __cookVacmAccessInfo(snmpEngine, groupName,
                                                    contextName, securityModel,
                                                    securityLevel)
+
+    delContext(snmpEngine, contextName)
+
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmAccessEntry.name + (9,) + tblIdx, 'destroy'),)
+        (vacmAccessEntry.name + (9,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -489,14 +523,16 @@ def addVacmView(snmpEngine, viewName, viewType, subTree, mask):
     vacmViewTreeFamilyEntry, tblIdx = __cookVacmViewInfo(snmpEngine, viewName,
                                                          subTree)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'destroy'),)
+        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmViewTreeFamilyEntry.name + (1,) + tblIdx, viewName),
-         (vacmViewTreeFamilyEntry.name + (2,) + tblIdx, subTree),
-         (vacmViewTreeFamilyEntry.name + (3,) + tblIdx, mask),
-         (vacmViewTreeFamilyEntry.name + (4,) + tblIdx, viewType),
-         (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'createAndGo'))
+        (vacmViewTreeFamilyEntry.name + (1,) + tblIdx, viewName),
+        (vacmViewTreeFamilyEntry.name + (2,) + tblIdx, subTree),
+        (vacmViewTreeFamilyEntry.name + (3,) + tblIdx, mask),
+        (vacmViewTreeFamilyEntry.name + (4,) + tblIdx, viewType),
+        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -504,7 +540,8 @@ def delVacmView(snmpEngine, viewName, subTree):
     vacmViewTreeFamilyEntry, tblIdx = __cookVacmViewInfo(snmpEngine, viewName,
                                                          subTree)
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'destroy'),)
+        (vacmViewTreeFamilyEntry.name + (6,) + tblIdx, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -558,40 +595,6 @@ def delVacmUser(snmpEngine, securityModel, securityName, securityLevel,
         )
 
 
-# Obsolete shortcuts for add/delVacmUser() wrappers
-
-def addRoUser(snmpEngine, securityModel, securityName, securityLevel, subTree):
-    addVacmUser(snmpEngine, securityModel, securityName,
-                securityLevel, subTree)
-
-
-def delRoUser(snmpEngine, securityModel, securityName, securityLevel, subTree):
-    delVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                subTree)
-
-
-def addRwUser(snmpEngine, securityModel, securityName, securityLevel, subTree):
-    addVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                subTree, subTree)
-
-
-def delRwUser(snmpEngine, securityModel, securityName, securityLevel, subTree):
-    delVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                subTree, subTree)
-
-
-def addTrapUser(snmpEngine, securityModel, securityName,
-                securityLevel, subTree):
-    addVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                (), (), subTree)
-
-
-def delTrapUser(snmpEngine, securityModel, securityName,
-                securityLevel, subTree):
-    delVacmUser(snmpEngine, securityModel, securityName, securityLevel,
-                (), (), subTree)
-
-
 # Notification target setup
 
 def __cookNotificationTargetInfo(snmpEngine, notificationName, paramsName,
@@ -629,33 +632,39 @@ def addNotificationTarget(snmpEngine, notificationName, paramsName,
                                              paramsName, filterSubtree)
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyEntry.name + (5,) + tblIdx1, 'destroy'),)
+        (snmpNotifyEntry.name + (5,) + tblIdx1, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyEntry.name + (2,) + tblIdx1, transportTag),
-         (snmpNotifyEntry.name + (3,) + tblIdx1, notifyType),
-         (snmpNotifyEntry.name + (5,) + tblIdx1, 'createAndGo'))
+        (snmpNotifyEntry.name + (2,) + tblIdx1, transportTag),
+        (snmpNotifyEntry.name + (3,) + tblIdx1, notifyType),
+        (snmpNotifyEntry.name + (5,) + tblIdx1, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'destroy'),)
+        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterProfileEntry.name + (1,) + tblIdx2, profileName),
-         (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'createAndGo'))
+        (snmpNotifyFilterProfileEntry.name + (1,) + tblIdx2, profileName),
+        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     if not snmpNotifyFilterEntry:
         return
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'destroy'),)
+        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterEntry.name + (1,) + tblIdx3, filterSubtree),
-         (snmpNotifyFilterEntry.name + (2,) + tblIdx3, filterMask),
-         (snmpNotifyFilterEntry.name + (3,) + tblIdx3, filterType),
-         (snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'createAndGo'))
+        (snmpNotifyFilterEntry.name + (1,) + tblIdx3, filterSubtree),
+        (snmpNotifyFilterEntry.name + (2,) + tblIdx3, filterMask),
+        (snmpNotifyFilterEntry.name + (3,) + tblIdx3, filterType),
+        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'createAndGo'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
@@ -667,18 +676,21 @@ def delNotificationTarget(snmpEngine, notificationName, paramsName,
                                              paramsName, filterSubtree)
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyEntry.name + (5,) + tblIdx1, 'destroy'),)
+        (snmpNotifyEntry.name + (5,) + tblIdx1, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'destroy'),)
+        (snmpNotifyFilterProfileEntry.name + (3,) + tblIdx2, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
     if not snmpNotifyFilterEntry:
         return
 
     snmpEngine.msgAndPduDsp.mibInstrumController.writeVars(
-        ((snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'destroy'),)
+        (snmpNotifyFilterEntry.name + (5,) + tblIdx3, 'destroy'),
+        **dict(snmpEngine=snmpEngine)
     )
 
 
