@@ -1,7 +1,7 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 import sys
@@ -18,28 +18,6 @@ getNextHandle = nextid.Integer(0x7fffffff)
 __null = univ.Null('')
 
 
-def getNextVarBinds(varBinds, origVarBinds=None):
-    errorIndication = None
-    idx = nonNulls = len(varBinds)
-    rspVarBinds = []
-    while idx:
-        idx -= 1
-        if varBinds[idx][1].tagSet in (rfc1905.NoSuchObject.tagSet,
-                                       rfc1905.NoSuchInstance.tagSet,
-                                       rfc1905.EndOfMibView.tagSet):
-            nonNulls -= 1
-        elif origVarBinds is not None:
-            if v2c.ObjectIdentifier(origVarBinds[idx][0]).asTuple() >= varBinds[idx][0].asTuple():
-                errorIndication = errind.oidNotIncreasing
-
-        rspVarBinds.insert(0, (varBinds[idx][0], __null))
-
-    if not nonNulls:
-        rspVarBinds = []
-
-    return errorIndication, rspVarBinds
-
-
 class CommandGenerator(object):
     _null = univ.Null('')
 
@@ -205,10 +183,6 @@ class CommandGenerator(object):
         return sendRequestHandle
 
 
-# backward compatibility stub
-CommandGeneratorBase = CommandGenerator
-
-
 class GetCommandGenerator(CommandGenerator):
     def processResponseVarBinds(self, snmpEngine, sendRequestHandle,
                                 errorIndication, PDU, cbCtx):
@@ -293,9 +267,7 @@ class NextCommandGenerator(NextCommandGeneratorSingleRun):
         elif not varBindTable:
             errorIndication, varBinds = errind.emptyResponse, ()
         else:
-            errorIndication, varBinds = getNextVarBinds(
-                varBindTable[-1], v2c.apiPDU.getVarBinds(reqPDU)
-            )
+            errorIndication, varBinds = v2c.apiPDU.getNextVarBinds(varBindTable[-1])
 
         if not cbFun(snmpEngine, sendRequestHandle, errorIndication,
                      v2c.apiPDU.getErrorStatus(PDU),
@@ -375,9 +347,7 @@ class BulkCommandGenerator(BulkCommandGeneratorSingleRun):
         elif not varBindTable:
             errorIndication, varBinds = errind.emptyResponse, ()
         else:
-            errorIndication, varBinds = getNextVarBinds(
-                varBindTable[-1], v2c.apiPDU.getVarBinds(reqPDU)
-            )
+            errorIndication, varBinds = v2c.apiBulkPDU.getNextVarBinds(varBindTable[-1])
             nonRepeaters = v2c.apiBulkPDU.getNonRepeaters(reqPDU)
             if nonRepeaters:
                 varBinds = v2c.apiBulkPDU.getVarBinds(reqPDU)[:int(nonRepeaters)] + varBinds[int(nonRepeaters):]
@@ -410,38 +380,3 @@ class BulkCommandGenerator(BulkCommandGeneratorSingleRun):
                     sendRequestHandle, statusInformation))
             cbFun(snmpEngine, sendRequestHandle,
                   statusInformation['errorIndication'], 0, 0, (), cbCtx)
-
-
-#
-# Obsolete, compatibility interfaces.
-#
-
-def __sendReqCbFun(snmpEngine, sendRequestHandle, errorIndication,
-                   errorStatus, errorIndex, varBinds, cbCtx):
-    cbFun, cbCtx = cbCtx
-    return cbFun(sendRequestHandle, errorIndication, errorStatus,
-                 errorIndex, varBinds, cbCtx)
-
-
-def _sendReq(self, snmpEngine, targetName, varBinds, cbFun,
-             cbCtx=None, contextEngineId=None, contextName=''):
-    return self.sendVarBinds(snmpEngine, targetName, contextEngineId,
-                             contextName, varBinds, __sendReqCbFun,
-                             (cbFun, cbCtx))
-
-
-def _sendBulkReq(self, snmpEngine, targetName, nonRepeaters, maxRepetitions,
-                 varBinds, cbFun, cbCtx=None, contextEngineId=None,
-                 contextName=''):
-    return self.sendVarBinds(snmpEngine, targetName, contextEngineId,
-                             contextName, nonRepeaters, maxRepetitions,
-                             varBinds, __sendReqCbFun, (cbFun, cbCtx))
-
-
-# install compatibility wrappers
-GetCommandGenerator.sendReq = _sendReq
-SetCommandGenerator.sendReq = _sendReq
-NextCommandGenerator.sendReq = _sendReq
-NextCommandGeneratorSingleRun.sendReq = _sendReq
-BulkCommandGenerator.sendReq = _sendBulkReq
-BulkCommandGeneratorSingleRun.sendReq = _sendBulkReq
