@@ -1,7 +1,7 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 import sys
@@ -23,7 +23,6 @@ class NotificationReceiver(object):
         )
 
         self.__snmpTrapCommunity = ''
-        self.__cbFunVer = 0
         self.__cbFun = cbFun
         self.__cbCtx = cbCtx
 
@@ -101,15 +100,5 @@ class NotificationReceiver(object):
             'processPdu: stateReference %s, user cbFun %s, cbCtx %s, varBinds %s' % (
                 stateReference, self.__cbFun, self.__cbCtx, varBinds))
 
-        if self.__cbFunVer:
-            self.__cbFun(snmpEngine, stateReference, contextEngineId,
-                         contextName, varBinds, self.__cbCtx)
-        else:
-            # Compatibility stub (handle legacy cbFun interface)
-            try:
-                self.__cbFun(snmpEngine, contextEngineId, contextName,
-                             varBinds, self.__cbCtx)
-            except TypeError:
-                self.__cbFunVer = 1
-                self.__cbFun(snmpEngine, stateReference, contextEngineId,
-                             contextName, varBinds, self.__cbCtx)
+        self.__cbFun(snmpEngine, stateReference, contextEngineId,
+                     contextName, varBinds, self.__cbCtx)
