@@ -1,7 +1,7 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 
@@ -175,13 +175,13 @@ authenticationFailure = AuthenticationFailure('Authenticator mismatched')
 class UnsupportedAuthProtocol(ErrorIndication):
     pass
 
-unsupportedAuthProtocol = UnsupportedAuthProtocol('Authentication protocol is not supprted')
+unsupportedAuthProtocol = UnsupportedAuthProtocol('Authentication protocol is not supported')
 
 
 class UnsupportedPrivProtocol(ErrorIndication):
     pass
 
-unsupportedPrivProtocol = UnsupportedPrivProtocol('Privacy protocol is not supprted')
+unsupportedPrivProtocol = UnsupportedPrivProtocol('Privacy protocol is not supported')
 
 
 class UnknownSecurityName(ErrorIndication):
@@ -199,9 +199,6 @@ unsupportedSecurityModel = UnsupportedSecurityModel('Unsupported SNMP security m
 class UnsupportedSecurityLevel(ErrorIndication):
     pass
 
-# backward compatibility plug
-UnsupportedSecLevel = UnsupportedSecurityLevel
-
 unsupportedSecurityLevel = UnsupportedSecurityLevel('Unsupported SNMP security level')
 
 
