@@ -1,19 +1,10 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 import random
-from pysnmp.proto.secmod.rfc3414.priv import base
-from pysnmp.proto.secmod.rfc3414.auth import hmacmd5, hmacsha
-from pysnmp.proto.secmod.rfc3414 import localkey
-from pysnmp.proto.secmod.rfc7860.auth import hmacsha2
-from pysnmp.proto import errind, error
-from pyasn1.type import univ
-from pyasn1.compat.octets import null
-from math import ceil
-
 try:
     from hashlib import md5, sha1
 except ImportError:
@@ -24,9 +15,19 @@ except ImportError:
     sha1 = sha.new
 
 try:
-    from Cryptodome.Cipher import DES3
+    from pysnmpcrypto import des3, PysnmpCryptoError
+
 except ImportError:
-    DES3 = None
+    PysnmpCryptoError = AttributeError
+    des3 = None
+
+from pysnmp.proto.secmod.rfc3414.priv import base
+from pysnmp.proto.secmod.rfc3414.auth import hmacmd5, hmacsha
+from pysnmp.proto.secmod.rfc3414 import localkey
+from pysnmp.proto.secmod.rfc7860.auth import hmacsha2
+from pysnmp.proto import errind, error
+from pyasn1.type import univ
+from pyasn1.compat.octets import null
 
 random.seed()
 
@@ -68,6 +69,13 @@ class Des3(base.AbstractEncryptionService):
                 'Unknown auth protocol %s' % (authProtocol,)
             )
         localPrivKey = localkey.localizeKey(privKey, snmpEngineID, hashAlgo)
+
+        # now extend this key if too short by repeating steps that includes the hashPassphrase step
+        while len(localPrivKey) < self.keySize:
+            # this is the difference between reeder and bluementhal
+            newKey = localkey.hashPassphrase(localPrivKey, hashAlgo)
+            localPrivKey += localkey.localizeKey(newKey, snmpEngineID, hashAlgo)
+
         return localPrivKey[:self.keySize]
 
     # 5.1.1.1
@@ -106,32 +114,28 @@ class Des3(base.AbstractEncryptionService):
 
     # 5.1.1.2
     def encryptData(self, encryptKey, privParameters, dataToEncrypt):
-        if DES3 is None:
-            raise error.StatusInformation(
-                errorIndication=errind.encryptionError
-            )
-
         snmpEngineBoots, snmpEngineTime, salt = privParameters
 
         des3Key, salt, iv = self.__getEncryptionKey(
             encryptKey, snmpEngineBoots
         )
 
-        des3Obj = DES3.new(des3Key, DES3.MODE_CBC, iv)
-
         privParameters = univ.OctetString(salt)
 
         plaintext = dataToEncrypt + univ.OctetString((0,) * (8 - len(dataToEncrypt) % 8)).asOctets()
-        ciphertext = des3Obj.encrypt(plaintext)
+
+        try:
+            ciphertext = des3.encrypt(plaintext, des3Key, iv)
+
+        except PysnmpCryptoError:
+            raise error.StatusInformation(
+                errorIndication=errind.unsupportedPrivProtocol
+            )
 
         return univ.OctetString(ciphertext), privParameters
 
     # 5.1.1.3
     def decryptData(self, decryptKey, privParameters, encryptedData):
-        if DES3 is None:
-            raise error.StatusInformation(
-                errorIndication=errind.decryptionError
-            )
         snmpEngineBoots, snmpEngineTime, salt = privParameters
 
         if len(salt) != 8:
@@ -146,9 +150,14 @@ class Des3(base.AbstractEncryptionService):
                 errorIndication=errind.decryptionError
             )
 
-        des3Obj = DES3.new(des3Key, DES3.MODE_CBC, iv)
-
         ciphertext = encryptedData.asOctets()
-        plaintext = des3Obj.decrypt(ciphertext)
+
+        try:
+            plaintext = des3.decrypt(ciphertext, des3Key, iv)
+
+        except PysnmpCryptoError:
+            raise error.StatusInformation(
+                errorIndication=errind.unsupportedPrivProtocol
+            )
 
         return plaintext
