@@ -1,7 +1,7 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 import os
@@ -16,7 +16,7 @@ try:
     from errno import ENOENT
 except ImportError:
     ENOENT = -1
-from pysnmp import version as pysnmp_version
+from pysnmp import __version__ as pysnmp_version
 from pysnmp.smi import error
 from pysnmp import debug
 
@@ -246,7 +246,7 @@ class MibBuilder(object):
     loadTexts = False
 
     # MIB modules can use this to select the features they can use
-    version = pysnmp_version
+    version = tuple([int(x) for x in pysnmp_version.split('.')])
 
     def __init__(self):
         self.lastBuildId = self._autoName = 0
@@ -290,21 +290,6 @@ class MibBuilder(object):
     def getMibSources(self):
         return tuple(self.__mibSources)
 
-    # Legacy/compatibility methods (won't work for .eggs)
-    def setMibPath(self, *mibPaths):
-        self.setMibSources(*[DirMibSource(x) for x in mibPaths])
-
-    def getMibPath(self):
-        paths = ()
-        for mibSource in self.getMibSources():
-            if isinstance(mibSource, DirMibSource):
-                paths += (mibSource.fullPath(),)
-            else:
-                raise error.MibLoadError(
-                    'MIB source is not a plain directory: %s' % (mibSource,)
-                )
-        return paths
-
     def loadModule(self, modName, **userCtx):
         for mibSource in self.__mibSources:
             debug.logger & debug.flagBld and debug.logger('loadModule: trying %s at %s' % (modName, mibSource))
