@@ -1,7 +1,7 @@
 #
 # This file is part of pysnmp software.
 #
-# Copyright (c) 2005-2017, Ilya Etingof <etingof@gmail.com>
+# Copyright (c) 2005-2018, Ilya Etingof <etingof@gmail.com>
 # License: http://snmplabs.com/pysnmp/license.html
 #
 import sys
@@ -506,7 +506,7 @@ class ObjectIdentity(object):
                         self.__oid += instIds
                         self.__indices = rowNode.getIndicesFromInstId(instIds)
                     except PyAsn1Error:
-                        raise SmiError('Instance index %r to OID convertion failure at object %r: %s' % (
+                        raise SmiError('Instance index %r to OID conversion failure at object %r: %s' % (
                             self.__args[2:], mibNode.getLabel(), sys.exc_info()[1]))
             elif self.__args[2:]:  # any other kind of MIB node with indices
                 if self.__args[2:]:
@@ -831,6 +831,8 @@ class ObjectType(object):
 
         Examples
         --------
+        >>> from pysmi.hlapi import varbinds
+        >>> mibViewController = varbinds.AbstractVarBinds.getMibViewController( engine )
         >>> objectType = ObjectType(ObjectIdentity('SNMPv2-MIB', 'sysDescr'), 'Linux i386')
         >>> objectType.resolveWithMib(mibViewController)
         ObjectType(ObjectIdentity('SNMPv2-MIB', 'sysDescr'), DisplayString('Linux i386'))
@@ -891,9 +893,8 @@ class NotificationType(object):
     Instances of :py:class:`~pysnmp.smi.rfc1902.NotificationType` class are
     containers incorporating :py:class:`~pysnmp.smi.rfc1902.ObjectIdentity`
     class instance (identifying particular notification) and a collection
-    of MIB variables IDs that
-    :py:class:`~pysnmp.entity.rfc3413.oneliner.cmdgen.NotificationOriginator`
-    should gather and put into notification message.
+    of MIB variables IDs that *NotificationOriginator* should gather
+    and put into notification message.
 
     Typical notification is defined like this (from *IF-MIB.txt*):
 
